
// Button.c
//====================================
// Differences between this function and the PicBasicPro
// compiler function:

// 1.  The PBP button function requires a "GOTO" label for
//     the last parameter. I didn't implement this feature
//     because while in Basic it's considered OK to use
//     GOTO statements, we don't normally use them in C.
//     Instead, I have the button() function return the
//     value of the "Action" parameter.  (1 or 0).
//     So if Action parameter is 1, and the button is
//     pressed, the return value is 1.   If the button
//     isn't pressed the function will return 0.

// 2.  The PBP button function has an internal debounce
//     delay in the auto-repeat code which can optionally
//     be disabled.  This internal delay isn't necessary
//     so I didn't include it in my version of the button
//     function.  The external delay explained in item 3
//     below, takes care of all debouncing. 

// 3.  A delay_ms(10) statement is required in the loop
//     in your main program that calls the button()
//     function.   Actually, PBP also requires this.
//     All their examples show a PAUSE 10 or PAUSE 100
//     statement in a loop.   But I'm explicitly stating
//     that it is required.

// 4.  The PBP function allows the Pin parameter to be
//     a number from 0-15 or a port.bit value.  I didn't do
//     it that way.  With the CCS compiler, we like to use
//     CCS pin numbers, which are defined in the .H file
//     for your PIC.   So for the pin parameter you must
//     use values such as PIN_B0, PIN_C1, PIN_E0, etc.
//     The pin parameter can be passed in a variable.
//     It doesn't have to be passed to the function as a
//     constant.

//  5.  The PBP function sets the TRIS for the specified pin
//      so it's an input.   Currently I don't do that, so you
//      have to set it with the set_tris_x() function.   If I
//      can decide on a clean way to do it, I may add that
//      feature later.

//====================================
// Description of function parameters:

// Pin:
// This must be a CCS-style pin number, such as PIN_B0.

// Downstate:
// This is the logic level of the button when it's pressed.
// For a circuit with a Normally-Open switch and a pull-up
// resistor, this parameter will be 0.

// Delay:
// This is the initial delay before auto-repeat begins.
// Example:  A Delay value of 50 means a 500 ms auto-repeat
// delay.

// Rate:
// This is the auto-repeat interval.
// Example:  A Rate value of 10 gives a 100 ms interval,
// which means an auto-repeat rate of 10 keys per second.     

// Bvar:
// This an 8-bit variable that must be declared in the main
// program, and it must be initialized to zero.  There must
// be a separate variable for each button that you use.
// Example: Declare the variable for pin B0 as int8 B0 = 0; 
// See the demo program for more examples.   Note that Bvar
// is a "reference variable" (it has a "&" in front of it).
// This is intentional.  It's done so the button function
// can directly access the Bvar variable which is declared
// outside the function, without having to pass a pointer
// to it.  This keeps the function interface simple.
//====================================

// Action:
// This is the value that's returned when the button is
// pressed.  Normally you set this = 1.       

// Return value:
// When the button is pressed, the value of the Action
// parameter will be returned.  When the button is not
// pressed, the opposite value will be returned.
// Normally, you set the Action parameter = 1, and so
// if the button is pressed, the function will return 1.
// If it's not pressed, it will return 0.  So you just
// poll the function every 10 ms in a loop, and check the
// return value with an if() statement.  If it's non-zero
// then the button is pressed and you can take appropriate
// action.


//=====================================
// The following macro is used by the Button function.
#define read_bit_var(x)  bit_test(*(int8 *)(x >> 3), x & 7)


//=====================================
int8 button(int16 pin, int8 downstate, int8 delay,
            int8 rate, int8 &BVar, int8 action)
{
int8 pin_value;

// Read the button pin.
pin_value = read_bit_var(pin);

// Check if the button is pressed.  It's pressed if the
// pin value is the same as the "downstate".   If it's not
// pressed, then zero the Bvar and return "Not pressed".
if(pin_value != downstate)
  {
   Bvar = 0;
   return(!action);   
  }

// The button is pressed.  Check to see if it's a new
// keypress.  We can tell if it's a new keypress by
// checking if BVar = 0.   If so, load the counter with
// the initial auto-repeat delay and return "Pressed".
// (If the delay has been set to 0, then load a non-zero
// value to allow the function to operate properly).
if(Bvar == 0)       
  {
   if(delay == 0)
      Bvar = 255;     
   else   
      Bvar = delay;   

   return(action); 
  }

// Decrement the auto-repeat counter. 
Bvar--;       

// Check if we just counted down to 0.   If so, then load
// the counter with the auto-repeat interval and return
// "Pressed".  If the delay is set to 0 or 255, it means
// that auto-repeat is disabled, so fall through and
// return "Not Pressed".
if(BVar == 0)   
  {
   BVar = rate;
   
   if((delay != 0) && (delay != 255))
      return(action);   
  }

// If the counter is positive, then it means an auto-repeat
// is still pending, so return "Not Pressed".
return(!action); 

}